# R codes used to generate gene set analysis results for evaluation of the gene set analysis methods
## Pashupati Mishra
## Liisa Holm's Bioinformatics lab
## Institute of Biotechnology, University of Helsinki
## 28.06.2016

load("brca_data_split.RData")
expr.data.brca <- brca_data$expr
gene.sets.brca <- brca_data$gene.sets
sample.labels <- brca_data$sample.labels


source("mGSZm.wrapper.R") 
source("std_allez_wrapper.R")

# gene set data in matrix form
gene.sets.brca.mat <- toMatrix(expr.data.brca,gene.sets.brca)

# gene set data in camera format
gene.sets.brca.camera <- list()
for(i in 1:ncol(gene.sets.brca.mat)){
    gene.sets.brca.camera[[i]] <- as.numeric(which(gene.sets.brca.mat[,i]==1))
}
names(gene.sets.brca.camera) <- colnames(gene.sets.brca.mat)

# gene set data in qusage format
gene.sets.brca.qusage <- list()
for(i in 1:ncol(gene.sets.brca.mat)){
    gene.sets.brca.qusage[[i]] <- rownames(expr.data.brca)[as.numeric(which(gene.sets.brca.mat[,i]==1))]
}

names(gene.sets.brca.qusage) <- colnames(gene.sets.brca.mat)


# gene set data in romer format
gene.sets.brca.romer <- list()
for(i in 1:ncol(gene.sets.brca.mat)){
    ind <- which(gene.sets.brca.mat[,i]==1)
    gene.sets.brca.romer[[i]] <- ind
}
names(gene.sets.brca.romer) <- colnames(gene.sets.brca.mat)


library(limma)
library(parallel)
library(doParallel)
library(foreach)
library(doMC)

n.cores <- detectCores()



cl <- makeCluster(50) 
registerDoParallel(cl)


# Generating random samples for 100 splits data
set.seed(345)
iter <- 100 
rand.ind <- vector("list",iter)
for(i in 1:iter){
  res <- matrix(NA,nrow=5,ncol=6)
  res[,1] <- sample(1:33,5,replace=F)
  res[,2] <- sample(34:38,5,replace=F)
  res[,3] <- sample(39:52,5,replace=F)
  res[,4] <- sample(53:75,5,replace=F)
  res[,5] <- sample(76:89,5,replace=F)
  res[,6] <- sample(90:94,5,replace=F)
  rand.ind[[i]] <- res
}

library(limma)

# sample labels
lbs_test <- unlist(lapply(c(1:6),function(x) rep(x,5)))
lbs_ref <- c(rep(1,28),rep(2,5),rep(3,9),rep(4,18),rep(5,9),rep(6,5))
lbs_camera_test <- unlist(lapply(letters[1:6],function(x) rep(x,5)))
lbs_camera_ref <- c(rep("a",28),rep("b",5),rep("c",9),rep("d",18),rep("e",9),rep("f",5))


# design and contrast matrix for reference data for camera
des_ref <- model.matrix(~0 +as.factor(lbs_camera_ref))
colnames(des_ref) <- levels(as.factor(lbs_camera_ref))
contr_ref <- makeContrasts(a-d,levels=des_ref)

# design and contrast matrix for test data for camera
des_test <- model.matrix(~0 +as.factor(lbs_camera_test))
colnames(des_test) <- levels(as.factor(lbs_camera_test))
contr_test <- makeContrasts(a-d,levels=des_test)



# generate splitted data with each random indices above

x <- foreach(i=1:iter) %dopar% {
    require(limma, quiet=TRUE)
    require(qusage, quiet=TRUE)
    require(gage, quiet=TRUE)
    ind <- as.numeric(rand.ind[[i]])
    expr_test <- expr.data.brca[,ind]
    ind_ref <- as.numeric(rand.ind[[i]][,c(1,3,4,5)])#because some will have 0 samples left if included all
    expr_ref <- expr.data.brca[,-ind_ref]
    
    source("mGSZm.wrapper.R") 
    source("std_allez_wrapper.R")
  
    # gene set analysis of ref data with mGSZm, mGSA, Allez and wKS (Perm1 and asymp p-value)
    gsz_ref <- mGSZm(expr.data=expr_ref,gene.sets=gene.sets.brca,sample.labels=lbs_ref,comparisons="group1-group4",method="m1_limma",perm.number=1000,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=TRUE,gsa.asymp=TRUE)
  
    mgsz_ref <- gsz_ref$comparison1$mGSZH
    mgsz_ref[,6] <- -log10(mgsz_ref[,6])
    mgsz_ref <- mgsz_ref[order(mgsz_ref[,6],mgsz_ref[,3],decreasing=T),]
    
    mgsa_ref <- gsz_ref$comparison1$mGSAH
    mgsa_ref[,6] <- -log10(mgsa_ref[,6])
    mgsa_ref <- mgsa_ref[order(mgsa_ref[,6],mgsa_ref[,3],decreasing=T),]
    
    allez_ref <- gsz_ref$comparison1$mAllezM
    allez_ref[,6] <- -log10(allez_ref[,6])
    allez_ref <- allez_ref[order(allez_ref[,6],allez_ref[,3],decreasing=T),]
    
    wks_ref <- gsz_ref$comparison1$wksM
    wks_ref[,6] <- -log10(wks_ref[,6])
    wks_ref <- wks_ref[order(wks_ref[,6],wks_ref[,3],decreasing=T),]
    
    
    # gene set analysis of ref data with camera
    camera_ref <- camera(expr_ref,gene.sets.brca.camera,design=des_ref,contrast=contr_ref)
  
    # gene set analysis of ref data with qusage
    qusage_ref <- qsTable(qusage(expr_ref,lbs_camera_ref, "a-d", gene.sets.brca.qusage), number=length(gene.sets.brca.qusage))

    # gene set analysis of ref data with gage 

    less <- gage(expr_ref, gsets = gene.sets.brca, ref = c(1:28), samp = c(43:60), compare = "unpaired",set.size=c(1,5000))$less
    greater <- gage(expr_ref, gsets = gene.sets.brca, ref = c(1:28), samp =c(43:60), compare = "unpaired",set.size=c(1,5000))$greater
    greater <- greater[order(rownames(greater)),]
    less <-less[order(rownames(less)),]
    gage_ref <- greater
    gage_ref[,3] <- pmin(greater[,3],less[,3])
    gage_ref <- gage_ref[order(gage_ref[,3]),]
  
    # gene set analysis of ref data with romer
  
    romer.temp <- romer(y=expr_ref,index=gene.sets.brca.romer,design=des_ref,contrast=contr_ref,nrot=1000)
    intermediate <- romer.temp[,c(2,3)]
    intermediate <- apply(intermediate,1,min)
    romer.temp <- cbind(romer.temp,intermediate)
    romer_ref <- romer.temp[order(romer.temp[,5],decreasing=F),]
    
    # gene set analysis of test data with eight different gene set analysis methods 
    # (mGSZm, mGSA, Allez, wKS, camera, qusage, gage and romer)
  
  
    # gene set analysis of test data with mGSZm
    mgsz.test <- mGSZm(expr.data=expr_test,gene.sets=gene.sets.brca,sample.labels=lbs_test,comparisons="group1-group4",m="m4_limma",perm.number=1000,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=TRUE,gsa.asymp=TRUE)$comparison1$mGSZH
    mgsz.test[,6] <- -log10(mgsz.test[,6])
    mgsz.test <- mgsz.test[order(mgsz.test[,6],mgsz.test[,3],decreasing=T),]
    
    # gene set analysis of test data with GSA*
    gsz.tmp.test <- mGSZm(expr.data=expr_test,gene.sets=gene.sets.brca,sample.labels=lbs_test,comparisons="group1-group4",method="m1_limma",perm.number=1000,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=FALSE,gsa.asymp=FALSE)
    mgsa.test <- gsz.tmp.test$comparison1$mGSAH
    mgsa.test[,6] <- -log10(mgsa.test[,6])
    mgsa.test <- mgsa.test[order(mgsa.test[,6],mgsa.test[,3],decreasing=T),]
    
    # gene set analysis of test data with wks
    wks.test <- gsz.tmp.test$comparison1$wksM
    wks.test[,6] <- -log10(wks.test[,6])
    wks.test <- wks.test[order(wks.test[,6],wks.test[,3],decreasing=T),]
  
    # gene set analysis of test data with camera
    camera.test <- camera(expr_test,gene.sets.brca.camera,design=des_test,contrast=contr_test)
    
    # gene set analysis of test data with qusage
    qusage.test <- qsTable(qusage(expr_test,lbs_camera_test, "a-d", gene.sets.brca.qusage), number=length(gene.sets.brca.qusage))
    
    
    # gene set analysis of test data with gage

    less <- gage(expr_test, gsets = gene.sets.brca, ref = c(1:5),samp=c(16:20), compare = "unpaired",set.size=c(1,5000))$less
    greater <- gage(expr_test, gsets = gene.sets.brca,ref=c(1:5),samp=c(16:20),compare = "unpaired",set.size=c(1,5000))$greater
    greater <- greater[order(rownames(greater)),]
    less <-less[order(rownames(less)),]
    gage.test <- greater
    gage.test[,3] <- pmin(greater[,3],less[,3])
    gage.test <- gage.test[order(gage.test[,3]),]
    
    # gene set analysis of test data with romer
    romer.test <- romer(y=expr_test,index=gene.sets.brca.romer,design=des_test,contrast=contr_test,nrot=1000)
    intermediate <- romer.test[,c(2,3)]
    intermediate <- apply(intermediate,1,min)
    romer.test <- cbind(romer.test,intermediate)
    romer.test <- romer.test[order(romer.test[,5],decreasing=F),]
    
    # gene set analysis of test data with standard allez
    std.allez.test <- allez1(expr_test,gene.sets.brca,lbs_test,comparisons="group1-group4",min.cl.sz=1,max.cl.sz=5000)
       return(list(mgsz_ref,mgsa_ref,allez_ref,wks_ref,camera_ref,qusage_ref,gage_ref,romer_ref,mgsz.test,mgsa.test,wks.test,camera.test,qusage.test,gage.test,romer.test,std.allez.test))
}


#########

source("generateGszCumsumDataBrca.R")
source("AvgGszCumsumPlotBrca.R")
methods.cumsumdata.brca <- generateGszCumsumDataBrca(x,top=c(3,5,7))
AvgGszCumsumPlotBrca(methods.cumsumdata.brca)




