## Pashupati Mishra
## Liisa Holm's Bioinformatics lab
## Institute of Biotechnology, University of Helsinki
## 28.06.2016
## Our version of R script for "Allez" R package that takes same inputs as that of mGSZ  ##

allez1 <- function(expr.data,gene.sets,sample.labels,comparisons, min.cl.sz,max.cl.sz){

gene.sets <- toMatrix(expr.data,gene.sets)

##Remove rows with no gene set members
data <- rm.rows.with.noSetMembers(expr.data,gene.sets)
gene.sets <- data$gene.sets
expr.data <- data$expr.data

##Remove too small gene sets
gene.sets <- filter.genesets(gene.sets,min.cl.sz, max.cl.sz)

gene_scores <- t_scores(expr.data,sample.labels,comparisons)
allez <- list()

for(j in 1:length(comparisons)){
    annotData <- setData(gene.sets,gene_scores[,j])
    allez[[j]] <- allez(gene_scores[,j],annotData)
    allez[[j]][,4] <- abs(allez[[j]][,4])
    allez[[j]] <- allez[[j]][order(allez[[j]][,4],decreasing=T),]
}
return(allez)
}




### subfunctions ###

allez <- function(scores,annotationData){
    
    #scores: differentially expressed gene scores
    
    set.mean <- tapply(annotationData$score,annotationData[[1]],mean,na.rm=TRUE,simplify=FALSE)
    set.sd <- unlist(tapply(annotationData$score,annotationData[[1]],sd,na.rm=TRUE,simplify=FALSE))
    set.size <- table(annotationData[[1]])
    class(set.size) <- "array"
    
    globe <- scores
    mu.globe <- mean(globe)
    sigma.globe <- sd(globe)
    G <- length(globe)
    
    dd <- sigma.globe * fact(G=G, m=set.size)
    z.score <- (as.numeric(set.mean) - mu.globe)/dd
    
                
    res <- cbind(data.frame(set.mean=set.mean, set.sd=set.sd,
    set.size=set.size)[names(z.score),],
    z.score=z.score)
    
    message("Labeling output ...")
    res
    
        
}

### subfunctions ###

fact <- function(G, m) sqrt(((G - m)/(G - 1))/m)

## sigma.fun <- function(m, E, esig2) {
##   vector <- c(m + 1/m - 2, (m - 1) * (m + 3/m - 2), -4 * (m - 1) * (1 - 1/m),
##               -2 * (m - 1) * (m - 2) * (1 - 3/m), (m - 1) * (m - 2) * (m - 3)/m)
##   sd.x <- sqrt((sum(vector * E)/((m - 1)^2) - esig2^2))
##   sd.x
## }

# This function replaces the one above. Notice this has one extra argument
sigma.fun <- function(m, E, G){
    Beta1 <- c(1, -3, -4, 12, -6)
    Beta2 <- c(0, 1, 0, -2, 1)
    var.x <- (1/m - 1/G)*Beta1 %*% E + (2/(m - 1) - 2/(G - 1))*Beta2 %*% E
    sd.x <- sqrt(var.x)
    sd.x
}

## R script to convert binary annotation data into set.data for implementation in standard "Allez" R package ##

setData <- function(annotationData,scores){
    setdata <- data.frame(NULL)
    for(i in 1:dim(annotationData)[1]){
        row <- annotationData[i,]
        classes <- names(which(row==1))
        genes <- rep(rownames(annotationData)[i],length(classes))
        score <- rep(scores[i],length(classes))
        genes_ind <- rep(i,length(classes))
        table <- data.frame(classes,genes_ind,genes,score)
        setdata <- rbind(setdata,table)
    }
    setdata
}

###

toMatrix <-
function(expr.data, gene.sets){
    if(is.matrix(gene.sets) | is.array(gene.sets) | is.data.frame(gene.sets) | is.table(gene.sets) ){
        gene.sets <- as.matrix(gene.sets)
    }
    
    if(is.list(gene.sets)){
        
            gene.set.matrix <- listTOclMatrix(rownames(expr.data),gene.sets)
            gene.set.size <- dim(gene.set.matrix)
            colnames(gene.set.matrix) <- names(gene.sets)
            rownames(gene.set.matrix) <- rownames(expr.data)
            gene.sets <- gene.set.matrix
            
        
    }
    return(gene.sets)
}

#############

listTOclMatrix <-
function(gene.vector, GO.list){
	num.classes <- length(GO.list)
	result <- list()
	for(i in 1:num.classes){
		result[[i]] <- gene.vector%in%GO.list[[i]]
	}
	out <- 1*(matrix(unlist(result),ncol=num.classes))
	return(out)
}

############

## removes too small and too big gene sets ##
filter.genesets <-
function(gene.sets,min.cl.sz, max.cl.sz){
    tmp_sum <- apply(gene.sets, 2, sum)
    ind_wth_sets <- which(tmp_sum > min.cl.sz & tmp_sum < max.cl.sz)
    gene.sets   <- gene.sets[,ind_wth_sets]
	out=gene.sets}

#############

### t-score calculation of compared groups ###
t_scores <- function(expr.data, sample.labels, comparisons){
  require(limma)
  require(Biobase)
  expr <- expr.data
  rownames(expr) <- NULL
  m <- gregexpr("[0-9]+",comparisons)
  CompGroups <- regmatches(comparisons,m)
  dime2 <- dim(expr)
  tscores.tot <- array(0, c(nrow(expr.data), length(CompGroups)))

  for(i in 1:length(CompGroups)){
    
    ## Calculation of t-scores for original data for a particular comparison ##
    design <- model.matrix(~0 + factor(sample.labels))
    colnames(design) <- paste("group",c(1:length(levels(factor(sample.labels)))),sep="")
    mySet <- new("ExpressionSet",expr=as.matrix(expr))
    fit1 <- lmFit(mySet, design)
    cont.matrix <- makeContrasts(contrasts=comparisons[i],levels=design)
    fit2 <- contrasts.fit(fit1, cont.matrix)
    fit2 <- eBayes(fit2)
    tscores.tot[,i] <- fit2$t
 }
   
    rownames(tscores.tot) <- rownames(expr.data)
    out <- tscores.tot
  
return(out)
  
}

##############

rm.rows.with.noSetMembers <-
function(expr.data,gene.sets){
	tmp_sum <- apply(gene.sets, 1, sum)
    ind_wth_sets <- which(tmp_sum > 0)
    expr.data <- expr.data[ind_wth_sets,]
    gene.sets   <- gene.sets[ind_wth_sets,]
    out <- list(expr.data=expr.data, gene.sets=gene.sets)
}
    
    
