# R codes used to generate gene set analysis results with breast cancer data
## Pashupati Mishra
## Liisa Holm's Bioinformatics lab
## Institute of Biotechnology, University of Helsinki
## 28.06.2016for evaluation of the permutation methods


load("brca_data_split.RData")

expr.data.brca <- brca_data$expr
gene.sets.brca <- brca_data$gene.sets
sample.labels <- brca_data$sample.labels
                
            
source("mGSZm.wrapper.R") 


# gene set data in matrix form
gene.sets.brca.mat <- toMatrix(expr.data.brca,gene.sets.brca)

# gene set data in camera format
gene.sets.brca.camera <- list()
for(i in 1:ncol(gene.sets.brca.mat)){
    gene.sets.brca.camera[[i]] <- as.numeric(which(gene.sets.brca.mat[,i]==1))
}
names(gene.sets.brca.camera) <- colnames(gene.sets.brca.mat)

# gene set data in qusage format
gene.sets.brca.qusage <- list()
for(i in 1:ncol(gene.sets.brca.mat)){
    gene.sets.brca.qusage[[i]] <- rownames(expr.data.brca)[as.numeric(which(gene.sets.brca.mat[,i]==1))]
}

names(gene.sets.brca.qusage) <- colnames(gene.sets.brca.mat)


# gene set data in romer format
gene.sets.brca.romer <- list()
for(i in 1:ncol(gene.sets.brca.mat)){
    ind <- which(gene.sets.brca.mat[,i]==1)
    gene.sets.brca.romer[[i]] <- ind
}
names(gene.sets.brca.romer) <- colnames(gene.sets.brca.mat)


library(limma)
library(parallel)
library(doParallel)
library(foreach)
library(doMC)

n.cores <- detectCores()

cl <- makeCluster(1) # 50 thiyo
registerDoParallel(cl)


# Generating random samples for 100 splits data
set.seed(789)
iter <- 1 # 100 thiyo
rand.ind <- vector("list",iter)
for(i in 1:iter){
  res <- matrix(NA,nrow=5,ncol=6)
  res[,1] <- sample(1:33,5,replace=F)
  res[,2] <- sample(34:38,5,replace=F)
  res[,3] <- sample(39:52,5,replace=F)
  res[,4] <- sample(53:75,5,replace=F)
  res[,5] <- sample(76:89,5,replace=F)
  res[,6] <- sample(90:94,5,replace=F)
  rand.ind[[i]] <- res
}



# sample labels
lbs_test <- unlist(lapply(c(1:6),function(x) rep(x,5)))
lbs_ref <- c(rep(1,28),rep(2,5),rep(3,9),rep(4,18),rep(5,9),rep(6,5))
lbs_camera_test <- unlist(lapply(letters[1:6],function(x) rep(x,5)))
lbs_camera_ref <- c(rep("a",28),rep("b",5),rep("c",9),rep("d",18),rep("e",9),rep("f",5))


# design and contrast matrix for reference data for camera
des_ref <- model.matrix(~0 +as.factor(lbs_camera_ref))
colnames(des_ref) <- levels(as.factor(lbs_camera_ref))
contr_ref <- makeContrasts(a-d,levels=des_ref)

# design and contrast matrix for test data for camera
des_test <- model.matrix(~0 +as.factor(lbs_camera_test))
colnames(des_test) <- levels(as.factor(lbs_camera_test))
contr_test <- makeContrasts(a-d,levels=des_test)


# generate splitted data with each random indices above

x <- foreach(i=1:iter) %dopar% {
    require(limma)
    require(Biobase)
    require(qusage)
    require(gage)
    require(ismev)
    ind <- as.numeric(rand.ind[[i]])
    expr_test <- expr.data.brca[,ind]
    ind_ref <- as.numeric(rand.ind[[i]][,c(1,3,4,5)])#because some will have 0 samples left if included all
    expr_ref <- expr.data.brca[,-ind_ref]
  
    # gene set analysis of ref data with mGSZm, GSA*, Allez and wKS (Perm1 and asymp p-value)
    gsz_ref <- mGSZm(expr.data=expr_ref,gene.sets=gene.sets.brca,sample.labels=lbs_ref,comparisons="group1-group4",method="m1_limma",perm.number=1000,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=TRUE,gsa.asymp=TRUE)
  
    mgsz_ref <- gsz_ref$comparison1$mGSZH
    mgsz_ref[,6] <- -log10(mgsz_ref[,6])
    mgsz_ref <- mgsz_ref[order(mgsz_ref[,6],mgsz_ref[,3],decreasing=T),]
    
    mgsa_ref <- gsz_ref$comparison1$mGSAH
    mgsa_ref[,6] <- -log10(mgsa_ref[,6])
    mgsa_ref <- mgsa_ref[order(mgsa_ref[,6],mgsa_ref[,3],decreasing=T),]
    
    allez_ref <- gsz_ref$comparison1$mAllezM
    allez_ref[,6] <- -log10(allez_ref[,6])
    allez_ref <- allez_ref[order(allez_ref[,6],allez_ref[,3],decreasing=T),]
    
    wks_ref <- gsz_ref$comparison1$wksM
    wks_ref[,6] <- -log10(wks_ref[,6])
    wks_ref <- wks_ref[order(wks_ref[,6],wks_ref[,3],decreasing=T),]
    
    
    # gene set analysis of ref data with camera
    camera_ref <- camera(expr_ref,gene.sets.brca.camera,design=des_ref,contrast=contr_ref)
  
    # gene set analysis of ref data with qusage
    qusage_ref <- qsTable(qusage(expr_ref,lbs_camera_ref, "a-d", gene.sets.brca.qusage), number=length(gene.sets.brca.qusage))

    # gene set analysis of ref data with gage 
    
    less <- gage(expr_ref, gsets = gene.sets.brca, ref = c(1:28), samp = c(43:60), compare = "unpaired",set.size=c(1,5000))$less
    greater <- gage(expr_ref, gsets = gene.sets.brca, ref = c(1:28), samp = c(43:60), compare = "unpaired",set.size=c(1,5000))$greater
    greater <- greater[order(rownames(greater)),]
    less <-less[order(rownames(less)),]
    gage_ref <- greater
    gage_ref[,3] <- pmin(greater[,3],less[,3])
    gage_ref <- gage_ref[order(gage_ref[,3]),]
  
    # gene set analysis of ref data with romer
  
    romer.temp <- romer(y=expr_ref,index=gene.sets.brca.romer,design=des_ref,contrast=contr_ref,nrot=1000)
    intermediate <- romer.temp[,c(2,3)]
    intermediate <- apply(intermediate,1,min)
    romer.temp <- cbind(romer.temp,intermediate)
    romer_ref <- romer.temp[order(romer.temp[,5],decreasing=F),]
    
    # gene set analysis of test data with gsz based on six different permutation methods
  
    perm1.tmp <- mGSZm(expr.data=expr_test,gene.sets=gene.sets.brca,sample.labels=lbs_test,comparisons="group1-group4",method="m1_limma",perm.number=1000,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=FALSE,gsa.asymp=FALSE)$comparison1$mGSZH
    #perm1.tmp[,6] <- -log10(perm1.tmp[,6])
    perm1.tmp <- perm1.tmp[order(perm1.tmp[,6],perm1.tmp[,3],decreasing=T),]
  
    perm2.tmp <- mGSZm(expr.data=expr_test,gene.sets=gene.sets.brca,sample.labels=lbs_test,comparisons="group1-group4",method="m2_limma",perm.number=1000,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=FALSE,gsa.asymp=FALSE)$comparison1$mGSZH
    #perm2.tmp[,6] <- -log10(perm2.tmp[,6])
    perm2.tmp <- perm2.tmp[order(perm2.tmp[,6],perm2.tmp[,3],decreasing=T),]
    
    perm3.tmp <- mGSZm(expr.data=expr_test,gene.sets=gene.sets.brca,sample.labels=lbs_test,comparisons="group1-group4",method="m4_limma",perm.number=1000,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=FALSE,gsa.asymp=FALSE)$comparison1$mGSZH
    #perm3.tmp[,6] <- -log10(perm3.tmp[,6])
    perm3.tmp <- perm3.tmp[order(perm3.tmp[,6],perm3.tmp[,3],decreasing=T),]
    
    perm4.tmp <- mGSZm(expr.data=expr_test,gene.sets=gene.sets.brca,sample.labels=lbs_test,comparisons="group1-group4",method="m3_limma",perm.number=1000,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=FALSE,gsa.asymp=FALSE)$comparison1$mGSZH
    #perm4.tmp[,6] <- -log10(perm4.tmp[,6])
    perm4.tmp <- perm4.tmp[order(perm4.tmp[,6],perm4.tmp[,3],decreasing=T),]
    
    perm5.tmp <- mGSZm(expr.data=expr_test,gene.sets=gene.sets.brca,sample.labels=lbs_test,comparisons="group1-group4",method="m44_limma",perm.number=1000,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=FALSE,gsa.asymp=FALSE)$comparison1$mGSZH
    #perm5.tmp[,6] <- -log10(perm5.tmp[,6])
    perm5.tmp <- perm5.tmp[order(perm5.tmp[,6],perm5.tmp[,3],decreasing=T),]
  
    perm6.tmp <- mGSZm(expr.data=expr_test,gene.sets=gene.sets.brca,sample.labels=lbs_test,comparisons="group1-group4",method="m33_limma",perm.number=1000,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=FALSE,gsa.asymp=FALSE)$comparison1$mGSZH
    #perm6.tmp[,6] <- -log10(perm6.tmp[,6])
    perm6.tmp <- perm6.tmp[order(perm6.tmp[,6],perm6.tmp[,3],decreasing=T),]
   
   return(list(mgsz_ref,mgsa_ref,allez_ref,wks_ref,camera_ref,qusage_ref,gage_ref,romer_ref,perm1.tmp,perm2.tmp,perm3.tmp,perm4.tmp,perm5.tmp,perm6.tmp))
}



#####

source("generatePermCumsumDataBrca.R")
perm.cumsumdata <- generatePermCumsumDataBrca(x,top=c(3,5,7))
source("AvgPermCumsumPlotBrca.R")
AvgPermCumsumPlotBrca(perm.cumsumdata)





