### R codes used for analysis of mouse tissue specific gene expression data
## Pashupati Mishra
## Liisa Holm's Bioinformatics lab
## Institute of Biotechnology, University of Helsinki
## 28.06.2016

# tissue_specific_gene_sets was generated as follows
# addition of random genes in tissue specific gene sets were as follows:

# 1. Each tissue specific gene set is used as a seed to generate 10 tissue 
#    specific gene sets with:
#	- 90% of the genes replaced by random genes
#	- 80% of the genes replaced by random genes
#	- 70% of the genes replaced by random genes
#	- 60% of the genes replaced by random genes
#	- 50% of the genes replaced by random genes
#	- 40% of the genes replaced by random genes
#	- 30% of the genes replaced by random genes
#	- 20% of the genes replaced by random genes
#	- 10% of the genes replaced by random genes
#	- 0% of the genes replaced by random genes

#2. The gene sets were named for example as, liver, liver1, liver2, liver3, liver4, liver5, liver6, liver7, liver8, liver9. liver has 0% of the genes replaced by random genes and liver9 has 90% of the genes replaced by random genes.


load("expr.data.mouse.RData")
load("mouse_gene_sets.RData")
load("tissue_specific_gene_sets.RData")
load("pos_sets_adipose_heart.RData")




# randomize the mouse_gene_sets 

source("randomize_genesets.R")

randomized_mouse_gene_sets <- randomize_genesets(mouse_gene_sets)


# merge the tissue_specific_gene_sets with randomized_mouse_gene_sets

mouse_analysis_gene_sets <- append(tissue_specific_gene_sets,randomized_mouse_gene_sets)


# Run mGSZm

source("mGSZH_final_empPval.R")

## "m3_limma" represents Perm4

lbs <- c(1,1,1,1,2,2,2,3,3,3,4,4,4,5,5,5,6,6,6) # sample labels
gsz_adipose_heart <- mGSZH_final_emp(expr.data=expr.data.mouse,gene.sets=mouse_analysis_gene_sets,sample.labels=lbs,comparisons="group5-group6",method="m3_limma",other.methods=TRUE,perm.number=2)

mgszm_adipose_heart <- gsz_adipose_heart$comparison1$mGSZH
mgszm_adipose_heart <- mgszm_adipose_heart[order(mgszm_adipose_heart[,6],decreasing=F),]

wks_adipose_heart <- gsz_adipose_heart$comparison1$wksM
wks_adipose_heart <- wks_adipose_heart[order(wks_adipose_heart[,6],decreasing=F),]


source("std_allez_wrapper.R")

std_allez_adipose_heart<-  allez1(expr.data.mouse,mouse_analysis_gene_sets,lbs,comparisons="group5-group6",min.cl.sz=1,max.cl.sz=5000)
std_allez_adipose_heart <- std_allez_adipose_heart[[1]]


st_gsa_adipose_heart <- mGSZH_final_emp(expr.data=expr.data.mouse,gene.sets=mouse_analysis_gene_sets,sample.labels=lbs,comparisons="group5-group6",method=="m1_limma",other.methods=TRUE,perm.number=2)

st_gsa_adipose_heart <- st_gsa_adipose_heart$comparison1$mGSAH
st_gsa_adipose_heart <- st_gsa_adipose_heart[order(st_gsa_adipose_heart[,6],st_gsa_adipose_heart[,3],decreasing=T),]


## romer (there are ties here but no second basis to rank them)

mouse_analysis_gene_sets_mat <- toMatrix(expr.data.mouse,mouse_analysis_gene_sets)
mouse_analysis_gene_sets_romer <- list()

for(i in 1:ncol(mouse_analysis_gene_sets_mat)){
 ind <- which(mouse_analysis_gene_sets_mat[,i]==1)
 mouse_analysis_gene_sets_romer[[i]] <- ind
}

names(mouse_analysis_gene_sets_romer) <- colnames(mouse_analysis_gene_sets_mat)

lbs_romer <- c(rep("a",4),rep("b",3),rep("c",3),rep("d",3),rep("e",3),rep("f",3))

design <- model.matrix(~0 + as.factor(lbs_romer))
colnames(design) <- levels(as.factor(lbs_romer))
contr <- makeContrasts(e-f,levels=design)
romer_adipose_heart <- romer(y=expr.data.mouse,index=mouse_analysis_gene_sets_romer,design=design,contrast=contr,nrot=1000)

intermediate <- romer_adipose_heart[,c(2,3)]
intermediate <- apply(intermediate,1,min)
romer_adipose_heart <- cbind(romer_adipose_heart,intermediate)
romer_adipose_heart <- romer_adipose_heart[order(romer_adipose_heart[,5],decreasing=F),]


## camera

mouse_analysis_gene_sets_camera <- list()

for(i in 1:ncol(mouse_analysis_gene_sets_mat)){
    mouse_analysis_gene_sets_camera[[i]] <- as.numeric(which(mouse_analysis_gene_sets_mat[,i]==1))
}
names(mouse_analysis_gene_sets_camera) <- colnames(mouse_analysis_gene_sets_mat)

camera_adipose_heart <- camera(expr.data.mouse,mouse_analysis_gene_sets_camera,design,contrast=contr)



## qusage 

library(qusage)
# gene set data in qusage format
mouse_analysis_gene_sets_qusage <- list()
for(i in 1:ncol(mouse_analysis_gene_sets_mat)){
    mouse_analysis_gene_sets_qusage[[i]] <- rownames(expr.data.mouse)[as.numeric(which(mouse_analysis_gene_sets_mat[,i]==1))]
}

names(mouse_analysis_gene_sets_qusage) <- colnames(mouse_analysis_gene_sets_mat)

qusage_adipose_heart <- qsTable(qusage(expr.data.mouse,lbs_romer, "e-f", mouse_analysis_gene_sets_qusage), number=length(mouse_analysis_gene_sets_qusage))


## gage
library(gage)
less <- gage(expr.data.mouse, gsets = mouse_analysis_gene_sets, ref = c(14:16), samp = c(17:19), compare = "unpaired",set.size=c(1,5000))$less
greater <- gage(expr.data.mouse, gsets = mouse_analysis_gene_sets, ref = c(14:16), samp = c(17:19), compare = "unpaired",set.size=c(1,5000))$greater
greater <- greater[order(rownames(greater)),]
less <-less[order(rownames(less)),]
gage_adipose_heart <- greater
gage_adipose_heart[,3] <- pmin(greater[,3],less[,3])
gage_adipose_heart <- gage_adipose_heart[order(gage_adipose_heart[,3]),]


## cumulative plot ##

source("mouse_plot.R")
plot_cumsum(mgszm_adipose_heart,st_gsa_adipose_heart,std_allez_adipose_heart,wks_adipose_heart,camera_adipose_heart,qusage_adipose_heart,gage_adipose_heart,romer_adipose_heart,pos_sets_adipose_heart)



## The above analysis was done for all 15 pairwise comparisons and plot based on average cumulative counts was generated.


