## mGSZm wrapper
## Pashupati Mishra
## Liisa Holm's Bioinformatics lab
## Institute of Biotechnology, University of Helsinki
## 28.06.2016

mGSZm <- function(expr.data,gene.sets,sample.labels,comparisons, method, min.cl.sz=1,max.cl.sz=5000, pre.var=0,wgt1=0.2,wgt2=0.5,var.constant=10,perm.number,gsz.asymp=TRUE, gsa.asymp=TRUE){



    
    start.val=5
    ## Testing that data sizes are correct #
   
   require(ismev)
   require(MASS)
    
    if(length(unique(sample.labels))==2){
        result <- mGSZ(expr.data=expr.data,gene.sets=gene.sets,sample.labels=sample.labels,perm.number=perm.number)
    }
    else{

   gene.sets <- toMatrix(expr.data,gene.sets)
   expr.dat.sz <- length(unique(rownames(expr.data)))
   num.genes <- length(unique(rownames(gene.sets)))
   
    ##Remove rows with no gene set members 
    
	data <- rm.rows.with.noSetMembers(expr.data,gene.sets)
	gene.sets <- data$gene.sets
    expr.data <- data$expr.data
    
    
     
   ##Remove too small gene sets
    
    gene.sets <- filter.genesets(gene.sets,min.cl.sz, max.cl.sz)
    geneset.classes <- colnames(gene.sets)
    num.classes <- ncol(gene.sets)
    geneset.class.sizes <- colSums(gene.sets)
    
    
   ##Calculation of gene scores

    if(method=="m1_limma"){
        print("m1_limma")
        tmp.expr.data = diffScoreMult1_limma(expr.data, gene.sets, labels=sample.labels, comparisons=comparisons, perm.number)
    }
        
        
 
   if(method=="m2_limma"){
    print("m2_limma")
        tmp.expr.data = diffScoreMult2_limma(expr.data, gene.sets, labels=sample.labels, comparisons=comparisons, perm.number)
    }
        
       
   if(method=="m3_limma"){
        print("m3_limma")
        tmp.expr.data = diffScoreMult3_limma(expr.data, gene.sets, labels=sample.labels, comparisons=comparisons, perm.number)
        
    }
        
        
   if(method=="m4_limma"){
        print("m4_limma")
        tmp.expr.data = diffScoreMult4_limma(expr.data, gene.sets, labels=sample.labels, comparisons=comparisons, perm.number)
    }
        
   if(method=="m33_limma"){
    print("m33_limma")
        tmp.expr.data = diffScoreMult33_limma(expr.data, gene.sets, labels=sample.labels, comparisons=comparisons, perm.number)
            
    }
    
   if(method=="m44_limma"){
    print("m44_limma")
            tmp.expr.data = diffScoreMult44_limma(expr.data, gene.sets, labels=sample.labels, comparisons=comparisons, perm.number)
    }
    
        
   diff.expr.dat.sz <- dim(tmp.expr.data[[1]])
    
    
    
    
    num.genes <- nrow(tmp.expr.data[[1]]) 
    pos.gsz <- list()
    pos.gsa <- list()
    pos.allez <- list()
    pos.wks <- list()
    
    
    for(i in 1:length(comparisons)){
        res <- mGSZ.test.score22(expr.data=tmp.expr.data[[i]][,1], gene.sets, wgt1, wgt2, pre.var, var.constant, start.val)
        pos.gsz[[i]] <- res$gene.set.scores$mGSZ.scores
        pos.gsa[[i]] <- res$gene.set.scores$mGSA.scores
        pos.allez[[i]] <- res$gene.set.scores$mAllez.scores
        pos.wks[[i]] <- KS.score(expr.data=tmp.expr.data[[i]][,1], gene.sets)$KS.mod
        
    }
    names(pos.gsz) <- comparisons
    names(pos.gsa) <- comparisons
    names(pos.allez) <- comparisons
    names(pos.wks) <- comparisons
    
  
    
	 ## mGSZ for sample permutation data ###
	
	col.perm.gsz <- list()
    col.perm.gsa <- list()
    col.perm.allez <- list()
    col.perm.wks <- list()
    
    
    
    for(j in 1:length(comparisons)){
        
        col.perm.gsz[[j]] <- matrix(0, perm.number, num.classes)
        col.perm.gsa[[j]] <- matrix(0, perm.number, num.classes)
        col.perm.allez[[j]] <- matrix(0, perm.number, num.classes)
        col.perm.wks[[j]] <- matrix(0, perm.number, num.classes)
    }

	
    for(l in 1:length(comparisons)){
        for( k in 1:perm.number){
            res <- mGSZ.test.score22(tmp.expr.data[[l]][,1+k], gene.sets,wgt1, wgt2, pre.var, var.constant, start.val)
            col.perm.gsz[[l]][k,] <- as.numeric(res$gene.set.scores$mGSZ.score)
            col.perm.gsa[[l]][k,] <- as.numeric(res$gene.set.scores$mGSA.score)
            col.perm.allez[[l]][k,] <- as.numeric(res$gene.set.scores$mAllez.score)
            col.perm.wks[[l]][k,] <- KS.score(tmp.expr.data[[l]][,1+k], gene.sets)$KS.mod
            

        }
    }
     
     


### p-value calculation for the gene set scores with sample permutation ###
    pvals.gsz <- list()
    pvals.gsa <- list()
    pvals.allez <- list()
    pvals.wks <- list()
    
    for(l in 1:length(comparisons)){
        if(gsz.asymp){
            print("GSZ scores with asymptotic p-value")
            pvals.gsz[[l]] <- mGSZ.p.values(pos.gsz[[l]],col.perm.gsz[[l]]) 
        }
        else{ 
            print("GSZ scores with empirical p-value")
            pvals.gsz[[l]] <- GSZ.p.values(pos.gsz[[l]],col.perm.gsz[[l]]) 
        }
        
        if(gsa.asymp){
            print("GSA scores with asymptotic p-value")
            pvals.gsa[[l]] <- mGSZ.p.values(pos.gsa[[l]],col.perm.gsa[[l]])
        }
        else{
            print("GSA scores with empirical p-value")
            pvals.gsa[[l]] <- GSZ.p.values(pos.gsa[[l]],col.perm.gsa[[l]])
        }
        
        pvals.allez[[l]] <- mAllez.p.values(pos.allez[[l]],col.perm.allez[[l]])
        pvals.wks[[l]] <- KS.p.values(pos.wks[[l]], col.perm.wks[[l]])
            
        }
    
    names(pvals.gsz) <- names(pos.gsz)
    names(pvals.gsa) <- names(pos.gsa)
    names(pvals.allez) <- names(pos.allez)
    names(pvals.wks) <- names(pos.wks)
    
    
    
    if(length(unique(rownames(expr.data)))<length(rownames(expr.data))){
        expdat <- aggregate(expr.data,by=list(rownames(expr.data)),FUN=mean)
        rownames(expdat) <- expdat[,1]
        expdat <- expdat[,2:dim(expdat)[2]] }
    else{expdat <- expr.data}

    ind <- match(rownames(tmp.expr.data[[1]]),rownames(expdat))
    expdat <- expdat[ind,]
    out1 <- expdat
    out2 <- gene.sets
    fc_scores <- gene_sets_fc_score(expdat, comparisons, sample.labels, gene.sets)
    
    ##Output table for each of the methods

    result <- list()
    for(a in 1:length(comparisons)){
        table.gsz <- data.frame(gene.sets = colnames(gene.sets)[pvals.gsz[[a]][[2]]], set.size = geneset.class.sizes[pvals.gsz[[a]][[2]]], Scores = pos.gsz[[a]][pvals.gsz[[a]][[2]]],FC = fc_scores[[a]][[2]][pvals.gsz[[a]][[2]]], abs_FC = fc_scores[[a]][[1]][pvals.gsz[[a]][[2]]], Pvalue = pvals.gsz[[a]][[1]],row.names=NULL)
        
        #table.gsz <- table.gsz[order(table.gsz[,6]),]
        
        table.gsa <- data.frame(gene.sets = colnames(gene.sets)[pvals.gsa[[a]][[2]]], set.size = geneset.class.sizes[pvals.gsa[[a]][[2]]], Scores = pos.gsa[[a]][pvals.gsa[[a]][[2]]],FC = fc_scores[[a]][[2]][pvals.gsa[[a]][[2]]], abs_FC = fc_scores[[a]][[1]][pvals.gsa[[a]][[2]]], Pvalue = pvals.gsa[[a]][[1]],row.names=NULL)
        
        #table.gsa <- table.gsa[order(table.gsa[,6])]
        
        table.allez <- data.frame(gene.sets = colnames(gene.sets)[pvals.allez[[a]][[2]]], set.size = geneset.class.sizes[pvals.allez[[a]][[2]]], Scores = pos.allez[[a]][pvals.allez[[a]][[2]]],FC = fc_scores[[a]][[2]][pvals.allez[[a]][[2]]], abs_FC = fc_scores[[a]][[1]][pvals.allez[[a]][[2]]], Pvalue = pvals.allez[[a]][[1]],row.names=NULL)
        
        #table.allez <- table.allez[order(table.allez[,6])]
        

        table.wks <- data.frame(gene.sets = colnames(gene.sets)[pvals.wks[[a]][[2]]], set.size = geneset.class.sizes[pvals.wks[[a]][[2]]], Scores = pos.wks[[a]][pvals.wks[[a]][[2]]],FC = fc_scores[[a]][[2]][pvals.wks[[a]][[2]]], abs_FC = fc_scores[[a]][[1]][pvals.wks[[a]][[2]]], Pvalue = pvals.wks[[a]][[1]],row.names=NULL)
            
            #table.wks <- table.wks[order(table.wks[,6]),]
        
        
        
        result[[a]] <- list(mGSZH = table.gsz, mGSAH = table.gsa, mAllezM = table.allez, wksM = table.wks)

    
    }
    names(result) <- paste("comparison",c(1:length(result)),sep="")
    }
    return(result)
}


#####################

### calculation of fold change of member genes ###
gene_sets_fc_score <- function(expr.data, comparisons, sample.labels, gene.sets){
  fc <- list()
  for(j in 1:length(comparisons)){
    grps <- getNumeric(comparisons[j])
    ind <- match(sample.labels,grps)
    ind.na <- which(is.na(ind))
    expr <- expr.data[,-ind.na]
    labels <- sample.labels[-ind.na]
    fc[[j]] <- expr.fc(expr, labels, gene.sets)
  }
  return(fc)
}

## R code to calculate mean fold change/absolute fold change of expression of member genes
expr.fc <- function(expr, labels, gene.sets){
  #expr.data: Subset of expr.data for a particular comparisons
  fc <- rep(NA,ncol(gene.sets))
  abs.fc <- fc
  grp1 <- table(labels)[[1]]
  grp2 <- table(labels)[[2]]
  for(i in 1:ncol(gene.sets)){
     ind <- which(gene.sets[,i]==1)
     exprdat <- expr[ind,]
     fc[i] <- -mean(apply(exprdat[,1:grp1],1,mean)-apply(exprdat[,(grp1+1):(grp1+grp2)],1,mean))
     abs.fc[i] <- mean(abs(apply(exprdat[,1:grp1],1,mean)-apply(exprdat[,(grp1+1):(grp1+grp2)],1,mean)))
     }
  out <- list(abs.fc=abs.fc,fc=fc)
  return(out)
}

### R code to get numeric part from a string ###
getNumeric <- function(string){
  str <- strsplit(string,"[^[:digit:]]")
  res <- as.numeric(unlist(str))
  sol <- res[!is.na(res)]
  return(sol)
}
#####################
###############
toTable <- function(mGSZMobj,comparisons){
    out <- mGSZMobj[[comparisons]]
    out <- out[order(out[,4],decreasing=F),]
    out
}

################

rm.rows.with.noSetMembers <-
function(expr.data,gene.sets){
	tmp_sum <- apply(gene.sets, 1, sum)
    ind_wth_sets <- which(tmp_sum > 0)
    expr.data <- expr.data[ind_wth_sets,]
    gene.sets   <- gene.sets[ind_wth_sets,]
    out <- list(expr.data=expr.data, gene.sets=gene.sets)}
    
###############
## removes too small and too big gene sets ##
filter.genesets <-
function(gene.sets,min.cl.sz, max.cl.sz){
    tmp_sum <- apply(gene.sets, 2, sum)
    ind_wth_sets <- which(tmp_sum > min.cl.sz & tmp_sum < max.cl.sz)
    gene.sets   <- gene.sets[,ind_wth_sets]
	out=gene.sets}

##################

sumVarMean_calc <-
function(expr_data, gene.sets, pre.var){
	dim_sets <- dim(gene.sets)
	length_expr <- length(expr_data)
	set_sz <- colSums(gene.sets)

	unique_class_sz <- unique(set_sz)
	num_genes <- length(expr_data)
	
	divider <- c(1:num_genes)
    mean_table <- cumsum(expr_data)/divider
    mean_table_sq <- mean_table^2
    var_table <- cumsum(expr_data^2)/divider - (mean_table)^2 + pre.var
    class_sz_index <- rep(0,dim_sets[2])
    for (i in 1:dim_sets[2]){
    	class_sz_index[i] <- which(set_sz[i]==unique_class_sz)
    }
    var_table <- var_table + pre.var
    hyge_stat <- count_hyge_var_mean(dim_sets[1],unique_class_sz)
    
    z_var <- matrix(numeric(0),dim_sets[1],length(unique_class_sz)) 
    
    z_mean <- z_var
    max_value <- 1:dim_sets[1]
    for (j in 1:length(unique_class_sz)){
    	prob_sum <- count.prob.sum(dim_sets[1],hyge_stat$mean[,j],hyge_stat$var[,j])
    	z_var[,j] <- 4*(var_table*prob_sum + mean_table_sq*hyge_stat$var[,j])
    	z_mean[,j] <- mean_table*(2*hyge_stat$mean[,j]-max_value)}
        
        out = list(Z_var = z_var, Z_mean = z_mean, class_size_index = class_sz_index,var_table=var_table,mean_table_sq=mean_table_sq,set_sz=set_sz)
        return(out)
  
    
}

#########

count.prob.sum <-
function(M, hyge.mean, hyge.var){
    tulos = matrix(rep(0,length(hyge.mean)),byrow=FALSE)
    N_tab = matrix(c(2:M),byrow=FALSE)
    tulos[1] = hyge.mean[1]
    tulos[2:M] = N_tab/(N_tab-1)*hyge.mean[2:M]-(hyge.mean[2:M]^2+hyge.var[2:M])/(N_tab-1)
    tulos = tulos
}

############

count_hyge_var_mean <-
function(M,K){
    N = matrix(rep((1:M),length(K)),ncol=length(K))
    K = matrix(rep(K,M),byrow=TRUE,ncol=length(K))
    out1 = N*K/M
    out2 = N*(K*(M-K)/M^2)*((M-N)/(M-1))
    results = list(mean= out1, var= out2)
}

##############

calc_z_var <-
function(num.genes,unique_class_sz_ln,pre_z_var,wgt2,var.constant){
	ones_matrix = matrix(1,num.genes,unique_class_sz_ln)
	ones_tmatrix = t(ones_matrix)
	median_matrix = apply(pre_z_var,2,median)
	pre_median_part = ones_tmatrix*median_matrix*wgt2
	median_part = t(pre_median_part)
	z_var = (pre_z_var + median_part + var.constant)^0.5
}

#############

geneSetsList <-
function(data){
  data <- GSA.read.gmt(data)
  geneSets <- data$genesets
  names(geneSets) <- data$geneset.names
  return(geneSets)
}


##############
diffScoreMult3_limma <- function(expr.data, gene.sets, labels, comparisons, perm.number){
    require(limma)
    require(Biobase)
    expr <- expr.data
    rownames(expr) <- NULL
    dime2 <- dim(expr)
    lbs <- list()
    u <- unique(labels)
    for(i in 1:(length(u))){
        lbs[[i]] <- which(labels==u[i])
    }
    perms <- vector("list",perm.number)
    for(j in 1:perm.number){
        lst <- list()
        lbs.org <- lbs
        lbs.ite <- lbs
        prbs <- unlist(lapply(lbs.org, function(x) sample(x, 1, replace=F))) ## vector of one sample from each grps
        lbs.ite <- updateLbs(lbs.ite,prbs)
        resamp <- function(x,...){if(length(x)==1) x else sample(x,...)}
        for(i in 1:length(lbs.org)){
            lbs.ite <- lbs.ite[sapply(lbs.ite, function(x) length(x)[1]) > 0] ##removing empty groups
            grps <- c(1:length(lbs.ite))[-i] ## vector of all the groups except ith group
            grp_sz <- length(lbs.org[[i]])-1 ## size of the grp samples which are to be sampled from grps
            if(length(grps)<grp_sz){
                x <- sample(unlist(lbs.ite),grp_sz,replace=F) ##this has to be improved
            }
            else{
            grp_ind <- sample(grps, grp_sz, replace=F) ## more groups than sample sz, so no of groups = grp_sz are sampled
            x <- unlist(lapply(lbs.ite[grp_ind],function(y) resamp(y, 1, replace=F)))}
            lst[[i]] <- c(prbs[i], x)
            lbs.ite <- updateLbs(lbs.ite,lst[[i]])
        }
    perms[[j]] <- lst
    }
    perm.designs <- getPointers(perms, labels)
    
        
        
        ## calculation of t-scores for original data ##########
    
        org.tscores <- list()
        design <- model.matrix(~0 + factor(labels))
        colnames(design) <- paste("group",c(1:length(levels(factor(labels)))),sep="")
        mySet <- new("ExpressionSet",expr=as.matrix(expr))
        fit1 <- lmFit(mySet, design)
        cont.matrix <- makeContrasts(contrasts=comparisons,levels=design)
        fit2 <- contrasts.fit(fit1, cont.matrix)
        fit2 <- eBayes(fit2)
        for(i in 1:length(comparisons)){
            org.tscores[[i]] <- fit2$t[,i]
        }
    
        ## Calculation of t-scores for permuted data for given comparisons ##
        perm.tscores <- vector("list",perm.number)
        for(k in 1:perm.number){
            perm.t <- list()
            design <- perm.designs[[k]]
            colnames(design) <- paste("group",c(1:length(levels(factor(labels)))),sep="")
            mySet <- new("ExpressionSet",expr=as.matrix(expr))
            fit1 <- lmFit(mySet, design)
            cont.matrix <- makeContrasts(contrasts=comparisons,levels=design)
            fit2 <- contrasts.fit(fit1, cont.matrix)
            fit2 <- eBayes(fit2)
            for(i in 1:length(comparisons)){
                perm.t[[i]] <- fit2$t[,i]
            }
            perm.tscores[[k]] <- perm.t
        }
    
        tscores.tot <- list()
        for(i in 1:length(comparisons)){
            tscores.tot[[i]] <- matrix(0,dime2[1],perm.number)
        }
    
        for(j in 1:length(comparisons)){
            for(k in 1:perm.number){
                tscores.tot[[j]][,k] <- perm.tscores[[k]][[j]]
            }
        }
    
        for(i in 1:length(comparisons)){
            tscores.tot[[i]] <- cbind(org.tscores[[i]], tscores.tot[[i]])
        }

return(tscores.tot)
}


#######################################################################

diffScoreMult4_limma <- function(expr.data, gene.sets, labels, comparisons=comparisons, perm.number){
    require(limma)
    require(Biobase)
    expr <- expr.data
    rownames(expr) <- NULL
    dime2 <- dim(expr)
    lbs <- list()
    u <- unique(labels)
    for(i in 1:(length(u))){
        lbs[[i]] <- which(labels==u[i])
    }
    perms <- vector("list",perm.number)
    for(j in 1:perm.number){
        lst <- list()
        lbs.org <- lbs
        lbs.ite <- lbs
        for(i in 1:length(lbs.org)){
                nonEmptyGrps <- sum(lapply(lbs.ite,function(x) length(x))!=0)
                if(nonEmptyGrps>=length(lbs.org[[i]])){
                    lst[[i]] <- c(resamp(unlist(lapply(lbs.ite,function(x) if(!length(x)==0){resamp(x,1,replace=F)})),length(lbs.org[[i]]),replace=F))}
                else{
                    lst[[i]] <- resamp(unlist(lbs.ite),length(lbs.org[[i]]),replace=F)}
                lbs.ite <- updateLbs(lbs.ite,lst[[i]])
        }
        perms[[j]] <- lst
    }
    
    perm.designs <- getPointers(perms, labels)
    
    
    org.tscores <- list()
    ###### calculation of t-scores for original data ##########
    design <- model.matrix(~0 + factor(labels))
    colnames(design) <- paste("group",c(1:length(levels(factor(labels)))),sep="")
    mySet <- new("ExpressionSet",expr=as.matrix(expr))
    fit1 <- lmFit(mySet, design)
    cont.matrix <- makeContrasts(contrasts=comparisons,levels=design)
    fit2 <- contrasts.fit(fit1, cont.matrix)
    fit2 <- eBayes(fit2)
    for(i in 1:length(comparisons)){
        org.tscores[[i]] <- fit2$t[,i]
    }
    
    ## Calculation of t-scores for permuted data for given comparisons ##
    perm.tscores <- vector("list",perm.number)
    for(k in 1:perm.number){
        perm.t <- list()
        design <- perm.designs[[k]]
        colnames(design) <- paste("group",c(1:length(levels(factor(labels)))),sep="")
        mySet <- new("ExpressionSet",expr=as.matrix(expr))
        fit1 <- lmFit(mySet, design)
        cont.matrix <- makeContrasts(contrasts=comparisons,levels=design)
        fit2 <- contrasts.fit(fit1, cont.matrix)
        fit2 <- eBayes(fit2)
        for(i in 1:length(comparisons)){
            perm.t[[i]] <- fit2$t[,i]
        }
        perm.tscores[[k]] <- perm.t
    }
    
    tscores.tot <- list()
    for(i in 1:length(comparisons)){
        tscores.tot[[i]] <- matrix(0,dime2[1],perm.number)
    }
    
    for(j in 1:length(comparisons)){
        for(k in 1:perm.number){
            tscores.tot[[j]][,k] <- perm.tscores[[k]][[j]]
        }
    }
    
    for(i in 1:length(comparisons)){
        tscores.tot[[i]] <- cbind(org.tscores[[i]], tscores.tot[[i]])
    }
    
    for(z in 1:length(comparisons)){
       rownames(tscores.tot[[z]]) <- rownames(expr.data)
       tscores.tot[[z]] <- aggregate(tscores.tot[[z]],by=list(rownames(tscores.tot[[z]])),FUN=mean)
       rownames(tscores.tot[[z]]) <- tscores.tot[[z]][,1]
       tscores.tot[[z]] <- tscores.tot[[z]][,2:dim(tscores.tot[[z]])[2]] 
       tscores.tot[[z]][order(match(rownames(tscores.tot[[z]]),rownames(gene.sets))),] ##thapeko
    }

    return(tscores.tot)
    
}


#######################################################################


diffScoreMult33_limma <- function(expr.data, gene.sets, labels, comparisons=comparisons, perm.number){
    require(limma)
    require(Biobase)
    expr <- expr.data
    rownames(expr) <- NULL
    dime2 <- dim(expr)
    lbs <- list()
    u <- unique(labels)
    for(i in 1:(length(u))){
        lbs[[i]] <- which(labels==u[i])
    }
    total <- list()
    for(c in 1:length(comparisons)){
        m <- gregexpr('[0-9]+',comparisons[c])
        grps <-  as.numeric(unlist(regmatches(comparisons[c],m)))
        grp1 <- matrix(0,nrow=perm.number,ncol=length(lbs[[grps[1]]]))
        grp2 <- matrix(0,nrow=perm.number,ncol=length(lbs[[grps[2]]]))

        for(j in 1:perm.number){
            lst <- list()
            lbs.org <- lbs
            
            ## the compared grps must also contribute to permutations##
            pg1 <- sample(lbs.org[[grps[1]]],2,replace=FALSE)
            pg2 <- sample(lbs.org[[grps[2]]],2,replace=FALSE)
            lbs.org <- lbs.org[-c(grps[1],grps[2])]
            g1 <- c(resamp(unlist(lapply(lbs.org,function(x) if(!length(x)==0){resamp(x,1,replace=F)})),(length(lbs[[grps[1]]])-2),replace=F))
            ind <- findInterval(match(g1, unlist(lbs.org)), cumsum(c(0,sapply(lbs.org, length)))+1)
            

                rel_grps <- lbs.org[ind]
        
                trans <- updateLbs(rel_grps,g1)
                g2 <- c(resamp(unlist(lapply(trans,function(x) if(!length(x)==0){resamp(x,1,replace=F)})),(length(lbs[[grps[2]]])-2),replace=F))
                g1 <- c(g1,pg1[1])
                g2 <- c(g2,pg1[2])
            
            
                g1 <- c(g1,pg2[1])
                g2 <- c(g2,pg2[2])
            
            
                grp1[j,] <- g1
                grp2[j,] <- g2
            }
        
    
        ###### calculation of t-scores for original data ##########
        design <- model.matrix(~0 + factor(labels))
        colnames(design) <- paste("group",c(1:length(levels(factor(labels)))),sep="")
        mySet <- new("ExpressionSet",expr=as.matrix(expr))
        fit1 <- lmFit(mySet, design)
        cont.matrix <- makeContrasts(contrasts=comparisons[c],levels=design)
        fit2 <- contrasts.fit(fit1, cont.matrix)
        fit2 <- eBayes(fit2)
        org.tscores <- fit2$t
        
    
        ## Calculation of t-scores for permuted data for given comparisons ##
        perm.t <- matrix(0,nrow=dim(expr.data)[1],ncol=perm.number)
        for(k in 1:perm.number){
            a=list()
            des <- model.matrix(~0 + factor(labels))
            des[,grps[1]] = 0
            des[,grps[2]] = 0
            des[grp1[k,], grps[1]] = 1
            des[grp2[k,], grps[2]] = 1
            colnames(des) <- paste("group",c(1:length(levels(factor(labels)))),sep="")
            mySet <- new("ExpressionSet",expr=as.matrix(expr))
            fit1 <- lmFit(mySet, des)
            cont.matrix <- makeContrasts(contrasts=comparisons[c],levels=des)
            fit2 <- contrasts.fit(fit1, cont.matrix)
            fit2 <- eBayes(fit2)
            perm.t[,k] <- fit2$t
        }
    out <- cbind(org.tscores,perm.t)
    rownames(out) <- rownames(expr.data)
    total[[c]] <- out
    }
    
return(total)

    
}


## function for sampling ##
resamp <- function(x,...){if(length(x)==1) x else sample(x,...)}

##function to update sample labels ##
updateLbs <- function(lbs,ind){
    l <- unlist(lbs)[-which(unlist(lbs)%in%ind)]
    lb <- list()
    for(i in 1:length(lbs)){
        lb[[i]] <- lbs[[i]][which(lbs[[i]]%in%l)]
    }
    lb
}


######################
## function to create design matrix for permuted data ##
getPointers <- function(perms,labels){
    ngroups <- length(unique(labels))
    perm.labels <- list()
    d <- matrix(0,length(labels),ngroups)
    for(k in 1:length(perms)){
        for(i in 1:ngroups){
            n <- rep(0,length(labels))
            n[perms[[k]][[i]]] <- 1
            d[,i] <- n
        }
        perm.labels[[k]] <- d
    }
    return(perm.labels)
}



######################

### t-score calculation based on the first option ###
diffScoreMult1_limma <- function(expr.data, gene.sets, labels, comparisons, perm.number){
  require(limma)
  require(Biobase)
  expr <- expr.data
  rownames(expr) <- NULL
  m <- gregexpr("[0-9]+",comparisons)
  CompGroups <- regmatches(comparisons,m)
  dime2 <- dim(expr)
  tscores.tot <- list()
  FC.tot <- tscores.tot
  #dime <- list()

  for(i in 1:length(CompGroups)){
    ## pit is length of compared groups combined
    pit <- length(c(labels[labels==levels(as.factor(labels))[as.numeric(CompGroups[[i]][1])]],labels[labels==levels(as.factor(labels))[as.numeric(CompGroups[[i]][2])]]))
    z <- c(labels[which(labels==levels(as.factor(labels))[as.numeric(CompGroups[[i]][1])])],labels[which(labels==levels(as.factor(labels))[as.numeric(CompGroups[[i]][2])])])
    perms <- replicate(perm.number,sample(z,pit,replace=FALSE))

    dime <- dim(perms)
    tscores.tot[[i]] <- array(0, c(dime2[1], dime[2] + 1))
    FC.tot[[i]] <- array(0, c(dime2[1], dime[2] + 1))
          
    ## Calculation of t-scores for original data for a particular comparison ##

    design <- model.matrix(~0 + factor(labels))
    colnames(design) <- paste("group",c(1:length(levels(factor(labels)))),sep="")
    mySet <- new("ExpressionSet",expr=as.matrix(expr))
    fit1 <- lmFit(mySet, design)
    cont.matrix <- makeContrasts(contrasts=comparisons[i],levels=design)
    fit2 <- contrasts.fit(fit1, cont.matrix)
    fit2 <- eBayes(fit2)
    tscores.tot[[i]][,1] <- fit2$t

    ## Calculation of t-scores for permuted data for a particular comparison ##

    for(k in 1:dime[2]){
      ind <- which(labels %in% CompGroups[[i]])
      lb <- labels
      lb[ind] <- perms[,k]
      design <- model.matrix(~0+as.factor(lb))
      colnames(design) <- paste("group",c(1:length(levels(factor(lb)))),sep="")
      fit1 <- lmFit(mySet, design)
      cont.matrix <- makeContrasts(contrasts=comparisons[i],levels=design)
      fit2 <- contrasts.fit(fit1, cont.matrix)
      fit2 <- eBayes(fit2)
      tscores.tot[[i]][,k+1] <- fit2$t
    }
    rownames(tscores.tot[[i]]) <- rownames(expr.data)
    if(length(unique(rownames(tscores.tot[[i]])))<length(rownames(tscores.tot[[i]]))){
        tscores.tot[[i]] <- aggregate(tscores.tot[[i]],by=list(rownames(tscores.tot[[i]])),FUN=mean)
        rownames(tscores.tot[[i]]) <- tscores.tot[[i]][,1]
        tscores.tot[[i]] <- tscores.tot[[i]][,2:dim(tscores.tot[[i]])[2]]
        ind <- match(rownames(gene.sets),rownames(tscores.tot[[i]]))
        tscores.tot[[i]] <- tscores.tot[[i]][ind,]
        }
      
out <- tscores.tot
  
return(out)
  }
}

######################

### t-score calculation based on the second option ###
diffScoreMult2_limma <- function(expr.data, gene.sets, labels, comparisons, perm.number,...){
  require(limma)
  require(Biobase)
  expr <- expr.data
  rownames(expr) <- NULL
  m <- gregexpr("[0-9]+",comparisons)
  CompGroups <- regmatches(comparisons,m)
  dime2 <- dim(expr)
  tscores.tot <- list()
  

  pit <- length(labels)
  perms <- replicate(perm.number,sample(labels,pit,replace=FALSE))
    

  dime <- dim(perms)
  
  for(i in 1:length(CompGroups)){
    tscores.tot[[i]] <- array(0, c(dime2[1], dime[2] + 1))


    ## Calculation of t-scores for original data for a particular comparison ##

    design <- model.matrix(~0 + as.factor(labels))
    colnames(design) <- paste("group",c(1:length(levels(as.factor(labels)))),sep="")
    mySet <- new("ExpressionSet",expr=as.matrix(expr))
    fit1 <- lmFit(mySet, design)
    cont.matrix <- makeContrasts(contrasts=comparisons[i],levels=design)
    fit2 <- contrasts.fit(fit1, cont.matrix)
    fit2 <- eBayes(fit2)
    tscores.tot[[i]][,1] <- fit2$t

    ## Calculation of t-scores for permuted data for a particular comparison ##

    for(k in 1:dime[2]){
      ind <- which(labels %in% CompGroups[[i]])
      lb <- perms[,k]
      design <- model.matrix(~0+as.factor(lb))
      colnames(design) <- paste("group",c(1:length(levels(as.factor(lb)))),sep="")
      fit1 <- lmFit(mySet, design)
      cont.matrix <- makeContrasts(contrasts=comparisons[i],levels=design)
      fit2 <- contrasts.fit(fit1, cont.matrix)
      fit2 <- eBayes(fit2)
      tscores.tot[[i]][,k+1] <- fit2$t
    }
    
    rownames(tscores.tot[[i]]) <- rownames(expr.data)
    if(length(unique(rownames(tscores.tot[[i]])))<length(rownames(tscores.tot[[i]]))){
        tscores.tot[[i]] <- aggregate(tscores.tot[[i]],by=list(rownames(tscores.tot[[i]])),FUN=mean)
        rownames(tscores.tot[[i]]) <- tscores.tot[[i]][,1]
        tscores.tot[[i]] <- tscores.tot[[i]][,2:dim(tscores.tot[[i]])[2]]
        tscores.tot[[i]][order(match(rownames(tscores.tot[[i]]),rownames(gene.sets))),] 
        
    }
  }

return(tscores.tot)
}


############################

toMatrix <-
function(expr.data, gene.sets){
  if(is.matrix(gene.sets) | is.array(gene.sets) | is.data.frame(gene.sets) | is.table(gene.sets) ){
    gene.sets <- as.matrix(gene.sets)
    }

  if(is.list(gene.sets)){

      gene.set.matrix <- listTOclMatrix(rownames(expr.data),gene.sets)
      gene.set.size <- dim(gene.set.matrix)
      colnames(gene.set.matrix) <- names(gene.sets)
      rownames(gene.set.matrix) <- rownames(expr.data)
      gene.sets <- gene.set.matrix
    
    
  }
  return(gene.sets)
}

#############

listTOclMatrix <-
function(gene.vector, GO.list){
	num.classes <- length(GO.list)
	result <- list()
	for(i in 1:num.classes){
		result[[i]] <- gene.vector%in%GO.list[[i]]
	}
	out <- 1*(matrix(unlist(result),ncol=num.classes))
	return(out)
}

#############

flipListStruct <- function(data){
    list_len <- length(data)
    max_val = 0
    ind_vals <- rep(0,list_len)
    loop_val <- 1
    for(k in 1:list_len){
        if(length(data[[k]])> 0){
            if(length(data[[k]])> 1 || data[[k]] > 0) {
                ind_vals[loop_val] <- k
                loop_val <- loop_val + 1
                max_val <- max(max_val, max(data[[k]]) )
            }
        }
        
    }
    ind_vals <- ind_vals[ind_vals > 0]
    out <- list(NULL)
    out[[max_val+1]] = 1
    out[[max_val+1]] = NULL 
    for(k in ind_vals){
        for(l in data[[k]] ){  
            out[[l]] <- c(out[[l]], k)
        }
    }
    out 
}

#############
mGSZ.test.score <-
function(expr.data, gene.sets, wgt1 = 0.2, wgt2 = 0.5, pre.var = 0, var.constant = 10, start.val = 5, table = FALSE,...){

    num.genes <- length(expr.data)

  # Ordering of gene expression data and gene sets data

  gene.sets <- toMatrix(expr.data,gene.sets)
  ord_out <- order(expr.data, decreasing= TRUE)
  expr.data <- expr.data[ord_out]
  set.dim <- dim(gene.sets)
  cols <- set.dim[2]
  gene.sets <- gene.sets[ord_out,]
 
    
  expr.data.ud <- expr.data[num.genes:1] # expression values turned up-side down
                                         # This does the analysis of the lower end


	num.genes=length(expr.data)
	num.classes=dim(gene.sets)[2]
	set_sz <- apply(gene.sets,2,sum)
	unique_class_sz_ln <- length(unique(set_sz))
	
	
	### Defining variables for different output

	mAllez <- rep(0,num.classes)
	mgsa <- matrix(0,2,num.classes)

	pre_z_var.1 <- sumVarMean_calc(expr.data, gene.sets, pre.var)
	pre_z_var.2 <- sumVarMean_calc(expr.data.ud, gene.sets, pre.var)
	
	Z_var1 = calc_z_var(num.genes,unique_class_sz_ln, pre_z_var.1$Z_var,wgt2,var.constant)
	Z_var2 = calc_z_var(num.genes,unique_class_sz_ln, pre_z_var.2$Z_var,wgt2,var.constant)

	out2 = matrix(0,2,num.classes)
	mgsa1 <- numeric(num.classes)
	mgsa2 <- mgsa1
	mgsz1 <- mgsa1
	mgsz2 <- mgsa2
	for (k in 1:num.classes){
		po1 <- which(gene.sets[,k]==1)
		po0 <- which(gene.sets[,k]==0)
		if(length(po1) > 0 & length(po0) > 0){
			tmp1 = expr.data
			tmp1[po0] = 0
			tmp0 = expr.data
			tmp0[po1] = 0
			result1 = cumsum(tmp1)-cumsum(tmp0)-pre_z_var.1$Z_mean[,pre_z_var.1$class_size_index[k]]
			result2 = cumsum(tmp1[num.genes:1])-cumsum(tmp0[num.genes:1])-pre_z_var.2$Z_mean[,pre_z_var.2$class_size_index[k]]
			
			result1[1:start.val] <- 0
			result2[1:start.val] <- 0
			A = result1/Z_var1[,pre_z_var.1$class_size_index[k]]
			B = result2/Z_var2[,pre_z_var.2$class_size_index[k]]
			mAllez[k] <- abs(A[num.genes])
			mgsa1[k] <- A[round(num.genes/2)]
			mgsa2[k] <- B[round(num.genes/2)]
			mgsz1[k] <- max(abs(A))
			mgsz2[k] <- max(abs(B))
			
	    }
    }
    mgsa[1,]<-mgsa1
    mgsa[2,]<-mgsa2
    out2[1,]<-mgsz1
    out2[2,]<-mgsz2
    if(table){
        out <- cbind(Gene.sets = colnames(gene.sets), mGSZ.scores = apply(out2,2,max), mAllez.scores = mAllez, mGSA.scores = apply(abs(mgsa),2,max))
    }
    else{
    out = list(mGSZ.scores = apply(out2,2,max), mAllez.scores = mAllez, mGSA.scores= apply(abs(mgsa),2,max))
    return(out)}
    
 }
#########

emp <-
function(pos.data, neg.data){
    
    neg.dat.sz <- dim(neg.data)
    test_data <- rep(0, neg.dat.sz[2])
    for (k in 1:neg.dat.sz[2]){
        test_data[k] <- length(which(neg.data[,k] >= pos.data[k]))
    }
    test_data[test_data == 0] <- 0.5
    test_data <- (test_data)/(neg.dat.sz[1])
    #test_data <- -log10(test_data)
    test_data
    
    
}

############

emp.wrs <-
function(pos.data, neg.data){
    neg.dat.sz <- dim(neg.data)
    test_data <- rep(0, neg.dat.sz[2])
    for (k in 1:neg.dat.sz[2]){
        test_data[k] <- length(which(neg.data[,k] >= abs(pos.data[k])))
    }
    test_data[test_data == 0] <- 0.5
    test_data <- (test_data)/(neg.dat.sz[1])
    #test_data <- -log10(test_data)
    test_data
}

###############

KS.p.values <-
function(pos.data, perm.data){
	#library(MASS)
	tmp <- mGSZ.adj.mean.std(pos.data, perm.data)
	pos.data <- tmp$pos.out
	perm.data <- tmp$perm.out
	rm(tmp)
	col.ind <- pick.data.cols(perm.data)
    perm.data <- perm.data[,col.ind]
	pos.data <- pos.data[col.ind]
	emp.pval.class <- emp(pos.data, perm.data)
		
    output <- list(EMP.class=emp.pval.class,class.ind=col.ind)
    
    
    return(output)
}

###############

logEVcdf <-
function(x, mu, sigma) {
    
    x <- (mu - x)/sigma  
    out <- rep(0, length(x))
    if(!(is.vector(x)) ){
        d.sz <- dim(x)
        out <- matrix(out, d.sz[1], d.sz[2])
    }
    po1 <- which(x < 5) 
    out[po1] <- -log(1 - exp(-exp(x[po1])) )
    x <- x[-po1]
    out[-po1] <- -x + exp(x)/2 - exp(2*x)/24 + exp(4*x)/2880 #Polyn. Expansion
    out <- out/log(10)
    out <- 10^(-out)
    out
}

###############

logNORMcdf <-
function(x,mu,sigma){
	z <- (x-mu)/(sigma)
	
	#out <- rep(0,length(x))
	if(!(is.vector(x)) ){
        d.sz <- dim(x)
        out <- matrix(out, d.sz[1], d.sz[2])
    }
  	out <- pnorm(-z,lower.tail=T)
  	#out <- -log10(out)
    
  	out
}

###############

mAllez.p.values <-
function(pos.data, perm.data){
	#library(MASS)
	tmp <- mGSZ.adj.mean.std(pos.data, perm.data)
	pos.data <- tmp$pos.out
	perm.data <- tmp$perm.out
	rm(tmp)
	col.ind <- pick.data.cols(perm.data)
	perm.data <- perm.data[,col.ind]
	pos.data <- pos.data[col.ind]
	norm.param.all <- fitdistr(as.numeric(perm.data),'normal')$estimate
	norm.pval.class <- rep(0,length(pos.data))
	
	for (k in 1:length(pos.data)){
        norm.param.class <- fitdistr(as.vector(perm.data[,k]),'normal')$estimate
    	norm.pval.class[k] <- logNORMcdf(pos.data[k],norm.param.class[1],norm.param.class[2])

	}
	

    output <- list(NORM.class=norm.pval.class,class.ind=col.ind)
    output
	
}

################

mGSA.p.values <-
function(pos.data, perm.data) {
    library(ismev)
    library(MASS)
    tmp <- mGSZ.adj.mean.std(pos.data, perm.data)
    scaled.pos.data <- tmp$pos.out
    scaled.perm.data <- tmp$perm.out
    rm(tmp)
    get.perm.p.vals = FALSE
    perm.dat.sz <- dim(scaled.perm.data)
    col.ind <- pick.data.cols(scaled.perm.data) # discard cols with null var

    # First fit the parameters to the whole data
    scaled.perm.data <- scaled.perm.data[,col.ind]
    scaled.pos.data <- scaled.pos.data[col.ind]
    ev.p.val.class  <- rep(0, length(scaled.pos.data))
        
    for (k in 1:length(scaled.pos.data)){
        
        # Following fits parameters to perms of each class
        ev.param.class  <- gum.fit(as.vector(scaled.perm.data[,k]),show=FALSE)$mle
        ev.p.val.class[k]  <- logEVcdf(scaled.pos.data[k], ev.param.class[1], ev.param.class[2]  )
        
        }
        output <- list(EV.class = ev.p.val.class,class.ind=col.ind)
        
        output
}

###############

mGSZ.p.values <-
function(pos.data, perm.data) {
    library(ismev)
    library(MASS)
    tmp <- mGSZ.adj.mean.std(pos.data, perm.data)
    pos.data <- tmp$pos.out
    perm.data <- tmp$perm.out
    rm(tmp)
    get.perm.p.vals = FALSE
    perm.dat.sz <- dim(perm.data)
    col.ind <- pick.data.cols(perm.data)
    perm.data <- perm.data[,col.ind]
    pos.data <- pos.data[col.ind]
    ev.p.val.class  <- rep(0,length(pos.data))
        
    
    for (k in 1:length(pos.data)){

        # Following fits parameters to perms of each class
        ev.param.class  <- gum.fit(as.vector(perm.data[,k]),show=FALSE)$mle
        ev.p.val.class[k]  <- logEVcdf(pos.data[k], ev.param.class[1], ev.param.class[2]  )
        }

        output <- list(EV.class = ev.p.val.class,class.ind=col.ind)
        
    
    output
}

####################

WRS.p.values <-
function(pos.data, perm.data){
	#library(MASS)
	tmp <- mGSZ.adj.mean.std(pos.data, perm.data)
	pos.data <- tmp$pos.out
	perm.data <- tmp$perm.out
	rm(tmp)
	col.ind <- pick.data.cols(perm.data)
    perm.data <- perm.data[,col.ind]
	pos.data <- pos.data[col.ind]
    emp.pval.class <- emp.wrs(pos.data, perm.data)
    norm.pval.class <- rep(0,length(pos.data))

    output <- list(EMP.class=emp.pval.class,class.ind=col.ind)
    
    output
	
}

################

mGSZ.adj.mean.std <-
function(pos.data, perm.data) {
   perm.sz <- dim(perm.data)
   mean.prof <- apply(perm.data, 2, mean)
   std.prof    <- apply(perm.data, 2, sd)
   null.ind <- which(std.prof == 0)  
   std.prof[null.ind] <- 0.1
   perm.out <- matrix(0, perm.sz[1], perm.sz[2])
   pos.out <- rep(0, perm.sz[2])
   for( k in 1:perm.sz[2]){
     perm.out[,k] <- ( perm.data[,k] - mean.prof[k]) /std.prof[k]
     pos.out[k]    <- ( pos.data[k] - mean.prof[k] )/std.prof[k]   
    }
   out  <- list(pos.out = pos.out, perm.out = perm.out)
   out
}

##############

pick.data.cols <-
function(data) {

    # Discard cols where var == 0
    # These disturb distribution fitting
    
    tmp <- apply(data, 2, var)
    out <- which(tmp > 0)
    out
}


## FC is computed with non-log signals ##

FC <- function(x, cl){
    x <- 2^x                    
    x.class1 <- x[(cl == 1)]    
    x.class2 <- x[(cl == 2)]    
    hoge <- log2(mean(x.class2)/mean(x.class1))
    return(hoge)
}

###############

mGSZ.test.score22 <-
function(expr.data, gene.sets, wgt1, wgt2, pre.var, var.constant,start.val,...){
	
  num.genes <- length(expr.data)

  # Ordering of gene expression data and gene sets data

  gene.sets <- toMatrix(expr.data,gene.sets)
  ord_out <- order(expr.data, decreasing= TRUE)
  expr.data <- expr.data[ord_out]
  set.dim <- dim(gene.sets)
  cols <- set.dim[2]
  gene.sets <- gene.sets[ord_out,] 
                           

 
  expr.data.ud <- expr.data[num.genes:1] # expression values turned up-side down
                                         # This does the analysis of the lower end


	num.genes=length(expr.data)
	num.classes=dim(gene.sets)[2]
	set_sz <- apply(gene.sets,2,sum)
	unique_class_sz_ln <- length(unique(set_sz))
	
	mAllez <- rep(0,num.classes)
    
	mgsa <- matrix(0,2,num.classes)

	pre_z_var.1 <- sumVarMean_calc(expr.data, gene.sets, pre.var)
	pre_z_var.2 <- sumVarMean_calc(expr.data.ud, gene.sets, pre.var)
	
	Z_var1 = calc_z_var(num.genes,unique_class_sz_ln, pre_z_var.1$Z_var,wgt2,var.constant)
	Z_var2 = calc_z_var(num.genes,unique_class_sz_ln, pre_z_var.2$Z_var,wgt2,var.constant)

	out2 = matrix(0,2,num.classes)
	mgsa1 <- numeric(num.classes)
	mgsa2 <- mgsa1
	mgsz1 <- mgsa1
	mgsz2 <- mgsa2
	for (k in 1:num.classes){
		po1 <- which(gene.sets[,k]==1)
		po0 <- which(gene.sets[,k]==0)
		if(length(po1) > 0 & length(po0) > 0){
			tmp1 = expr.data
			tmp1[po0] = 0
			tmp0 = expr.data
			tmp0[po1] = 0
			result1 = cumsum(tmp1)-cumsum(tmp0)-pre_z_var.1$Z_mean[,pre_z_var.1$class_size_index[k]]
			result2 = cumsum(tmp1[num.genes:1])-cumsum(tmp0[num.genes:1])-pre_z_var.2$Z_mean[,pre_z_var.2$class_size_index[k]]
			
			result1[1:start.val] <- 0
			result2[1:start.val] <- 0
			A = result1/Z_var1[,pre_z_var.1$class_size_index[k]]
			B = result2/Z_var2[,pre_z_var.2$class_size_index[k]]
           
            mAllez[k] <- abs(A[num.genes])
			mgsa1[k] <- A[round(num.genes/2)]
			mgsa2[k] <- B[round(num.genes/2)]
			mgsz1[k] <- max(abs(A))
			mgsz2[k] <- max(abs(B))
			
	    }
    }
    mgsa[1,]<-mgsa1
    mgsa[2,]<-mgsa2
    out2[1,]<-mgsz1
    out2[2,]<-mgsz2
    result1 = list(mGSZ.scores = apply(out2,2,max), mAllez.scores = mAllez, mGSA.scores= apply(abs(mgsa),2,max))
    result2 = list(Z_var1=Z_var1,Z_var2=Z_var2,Z_mean1=pre_z_var.1$Z_mean,Z_mean2=pre_z_var.2$Z_mean,class_size_index1=pre_z_var.1$class_size_index,class_size_index2=pre_z_var.2$class_size_index)
    
    out = list(gene.set.scores=result1, var.attributes=result2)
    return(out)
 }

#########################

diffScoreMult3_mlimma <- function(expr.data, gene.sets, labels, comparisons=comparisons, perm.number){
    require(limma)
    require(Biobase)
    expr <- expr.data
    rownames(expr) <- NULL
    dime2 <- dim(expr)
    lbs <- list()
    u <- unique(labels)
    for(i in 1:(length(u))){
        lbs[[i]] <- which(labels==u[i])
    }
    perms <- vector("list",perm.number)
    for(j in 1:perm.number){
        lst <- list()
        lbs.org <- lbs
        lbs.ite <- lbs
        prbs <- unlist(lapply(lbs.org, function(x) sample(x, 1, replace=F))) ## vector of one sample from each grps
        lbs.ite <- updateLbs(lbs.ite,prbs)
        resamp <- function(x,...){if(length(x)==1) x else sample(x,...)}
        for(i in 1:length(lbs.org)){
            lbs.ite <- lbs.ite[sapply(lbs.ite, function(x) length(x)[1]) > 0] ##removing empty groups
            grps <- c(1:length(lbs.ite))[-i] ## vector of all the groups except ith group
            grp_sz <- length(lbs.org[[i]])-1 ## size of the grp samples which are to be sampled from grps
            if(length(grps)<grp_sz){
                x <- sample(unlist(lbs.ite),grp_sz,replace=F)
            }
            else{
                grp_ind <- sample(grps, grp_sz, replace=F) ## more groups than sample sz, so no of groups = grp_sz are sampled
                x <- unlist(lapply(lbs.ite[grp_ind],function(y) resamp(y, 1, replace=F)))}
            lst[[i]] <- c(prbs[i], x)
            lbs.ite <- updateLbs(lbs.ite,lst[[i]])
        }
        perms[[j]] <- lst
    }
    perm.designs <- getPointers(perms, labels)
    
        
        org.tscores <- list()
        designnn <- model.matrix(~0 + factor(labels))
        colnames(designnn) <- paste("group",c(1:length(levels(factor(labels)))),sep="")
        for(i in 1:length(comparisons)){
            grp.ind <- getNumeric(comparisons[i])
            ind1 <- as.numeric(which(designnn[,grp.ind[1]]==1))
            ind2 <- as.numeric(which(designnn[,grp.ind[2]]==1))
            exprdat <- expr[,c(ind1,ind2)]
            lbs <- c(rep(1,length(ind1)),rep(2,length(ind2)))
            design <- model.matrix(~lbs)
            mySet <- new("ExpressionSet",expr=as.matrix(exprdat))
            fit1 <- lmFit(mySet, design)
            fit2 <- eBayes(fit1)
            org.tscores[[i]] <- fit2$t[,2]
        }
        
        ## Calculation of t-scores for permuted data for given comparisons ##
        perm.tscores <- vector("list",perm.number)
        for(k in 1:perm.number){
            perm.t <- list()
            designnn <- perm.designs[[k]]
            colnames(designnn) <- paste("group",c(1:length(levels(factor(labels)))),sep="")
            for(i in 1:length(comparisons)){
                grp.ind <- getNumeric(comparisons[i])
                ind1 <- as.numeric(which(designnn[,grp.ind[1]]==1))
                ind2 <- as.numeric(which(designnn[,grp.ind[2]]==1))
                exprdat <- expr[,c(ind1,ind2)]
                lbs <- c(rep(1,length(ind1)),rep(2,length(ind2)))
                design <- model.matrix(~lbs)
                mySet <- new("ExpressionSet",expr=as.matrix(exprdat))
                fit1 <- lmFit(mySet, design)
                fit2 <- eBayes(fit1)
                perm.t[[i]] <- fit2$t[,2]
            }
            perm.tscores[[k]] <- perm.t
        }
        
        tscores.tot <- list()
        for(i in 1:length(comparisons)){
            tscores.tot[[i]] <- matrix(0,dime2[1],perm.number)
        }
        
        for(j in 1:length(comparisons)){
            for(k in 1:perm.number){
                tscores.tot[[j]][,k] <- perm.tscores[[k]][[j]]
            }
        }
        
        for(i in 1:length(comparisons)){
            tscores.tot[[i]] <- cbind(org.tscores[[i]], tscores.tot[[i]])
        }
        
        for(z in 1:length(comparisons)){
            rownames(tscores.tot[[z]]) <- rownames(expr.data)
            if(length(unique(rownames(tscores.tot[[z]])))<length(rownames(tscores.tot[[z]]))){
                tscores.tot[[z]] <- aggregate(tscores.tot[[z]],by=list(rownames(tscores.tot[[z]])),FUN=mean)
                rownames(tscores.tot[[z]]) <- tscores.tot[[z]][,1]
                tscores.tot[[z]] <- tscores.tot[[z]][,2:dim(tscores.tot[[z]])[2]]
                tscores.tot[[z]][order(match(rownames(tscores.tot[[z]]),rownames(gene.sets))),] ##thapeko
            }
            
            ind <- match(rownames(gene.sets),rownames(tscores.tot[[z]]))
            tscores.tot[[z]] <- tscores.tot[[z]][ind,]
            
        }
        
        
        
        out <- tscores.tot
        
    }


#################################################################

diffScoreMult44_limma <- function(expr.data, gene.sets, labels, comparisons=comparisons, perm.number){
    require(limma)
    require(Biobase)
    expr <- expr.data
    rownames(expr) <- NULL
    dime2 <- dim(expr)
    lbs <- list()
    u <- unique(labels)
    for(i in 1:(length(u))){
        lbs[[i]] <- which(labels==u[i])
    }
    total <- list()
    for(c in 1:length(comparisons)){
        m <- gregexpr('[0-9]+',comparisons[c])
        grps <-  as.numeric(unlist(regmatches(comparisons[c],m)))
        grp1 <- matrix(0,nrow=perm.number,ncol=length(lbs[[grps[1]]]))
        grp2 <- matrix(0,nrow=perm.number,ncol=length(lbs[[grps[2]]]))
        #grp_list <- list()
        for(j in 1:perm.number){
            lst <- list()
            lbs.org <- lbs
            g1 <- c(resamp(unlist(lapply(lbs.org,function(x) if(!length(x)==0){resamp(x,1,replace=F)})),length(lbs.org[[grps[1]]]),replace=F))
            ind <- findInterval(match(g1, unlist(lbs)), cumsum(c(0,sapply(lbs, length)))+1)
            rel_grps <- lbs.org[ind]
            
            trans <- updateLbs(rel_grps,g1)
            g2 <- c(resamp(unlist(lapply(trans,function(x) if(!length(x)==0){resamp(x,1,replace=F)})),length(lbs.org[[grps[2]]]),replace=F))
            grp1[j,] <- g1
            grp2[j,] <- g2
        }
        
        
        
        
        ###### calculation of t-scores for original data ##########
        design <- model.matrix(~0 + factor(labels))
        colnames(design) <- paste("group",c(1:length(levels(factor(labels)))),sep="")
        mySet <- new("ExpressionSet",expr=as.matrix(expr))
        fit1 <- lmFit(mySet, design)
        cont.matrix <- makeContrasts(contrasts=comparisons[c],levels=design)
        fit2 <- contrasts.fit(fit1, cont.matrix)
        fit2 <- eBayes(fit2)
        org.tscores <- fit2$t
        
        
        ## Calculation of t-scores for permuted data for given comparisons ##
        perm.t <- matrix(0,nrow=dim(expr.data)[1],ncol=perm.number)
        for(k in 1:perm.number){
            a=list()
            des <- model.matrix(~0 + factor(labels))
            des[,grps[1]] = 0
            des[,grps[2]] = 0
            des[grp1[k,], grps[1]] = 1
            des[grp2[k,], grps[2]] = 1
            colnames(des) <- paste("group",c(1:length(levels(factor(labels)))),sep="")
            mySet <- new("ExpressionSet",expr=as.matrix(expr))
            fit1 <- lmFit(mySet, des)
            cont.matrix <- makeContrasts(contrasts=comparisons[c],levels=des)
            fit2 <- contrasts.fit(fit1, cont.matrix)
            fit2 <- eBayes(fit2)
            perm.t[,k] <- fit2$t
        }
        out <- cbind(org.tscores,perm.t)
        rownames(out) <- rownames(expr.data)
        total[[c]] <- out
    }
    
    return(total)
    
}

###########################################################
trim <- function (x) gsub("^\\s+|\\s+$", "", x)

#######

KS.score <-
function(expr.data, gene.sets){
    ord <- order(expr.data,decreasing=FALSE)
    expr.data <- expr.data[ord]
    gene.sets <- gene.sets[ord,]
    set.dim <- dim(gene.sets)
    result1 <- rep(0,set.dim[2])
    result2 <- result1
    profile2 <- matrix(0,set.dim[2],set.dim[1])
    expr.data.len <- length(expr.data)
    for(i in 1:set.dim[2]){
        po1 <- which(gene.sets[,i]==1)
        po0 <- which(gene.sets[,i]==0)
        tmp1 <- expr.data
        tmp1[po0] <- 0
        tmp0 <- expr.data
        tmp0[po1] <- 0
        tmp2 <- cumsum(abs(tmp1))/sum(abs(tmp1))
        tmp2b <- tmp1
        tmp2b[po1] <- 1
        tmp2c <- cumsum(tmp2b)/sum(tmp2b)
        tmp0b <- tmp0
        tmp0b[po0] <- 1
        tmp3  <- cumsum(abs(tmp0))/sum(abs(tmp0))
        tmp3b <- cumsum(abs(tmp0b))/sum(abs(tmp0b))
        result1[i]  = max(abs(tmp2c - tmp3b))
        result2[i] = max(abs(tmp2 - tmp3b))
        
    }
    
    out <- list(KS.org = result1,KS.mod = result2)
    
    
}

###########

emp <-
function(pos.data, neg.data){
    
    neg.dat.sz <- dim(neg.data)
    test_data <- rep(0, neg.dat.sz[2])
    for (k in 1:neg.dat.sz[2]){
        test_data[k] <- length(which(neg.data[,k] >= pos.data[k]))
    }
    test_data[test_data == 0] <- 0.5
    test_data <- (test_data)/(neg.dat.sz[1])
    #test_data <- -log10(test_data)
    test_data
    
    
}

GSZ.p.values <- function(pos.data, perm.data,add=1) {
    
    
    library(ismev)
    library(MASS)
    tmp <- GSZ.adj.mean.std(pos.data, perm.data)
    pos.data <- tmp$pos.out
    perm.data <- tmp$perm.out
    rm(tmp)
    get.perm.p.vals = FALSE
    perm.dat.sz <- dim(perm.data)
    col.ind <- pick.data.cols(perm.data) # discard cols with null var
    
    perm.data <- perm.data[,col.ind]
    pos.data <- pos.data[col.ind]
    
    
    div.length <- length(perm.data)
    
    emp.p.val.pool <- rep(0, length(pos.data))
    emp.p.val.sep <- get.class.emp.p_vals.gsz(pos.data, perm.data)
        for (k in 1:length(pos.data)){
        # Empirical p-value (against pooled permutations)
        
        divided <- length(which(perm.data >= pos.data[k]))
        emp.p.val.pool[k] <- log10(div.length+add) - log10(divided+add)
        
        
        
    }
  
    
    output <- list(EMP.class = emp.p.val.sep,class.ind=col.ind)
        
    
    output
}

GSZ.adj.mean.std <- function(pos.data, perm.data) {
    perm.sz <- dim(perm.data)
    mean.prof <- apply(perm.data, 2, mean)
    std.prof    <- apply(perm.data, 2, sd)
    null.ind <- which(std.prof == 0)   # Remove null dividers
    std.prof[null.ind] <- 0.1
    perm.out <- matrix(0, perm.sz[1], perm.sz[2])
    pos.out <- rep(0, perm.sz[2])
    for( k in 1:perm.sz[2]){
        perm.out[,k] <- ( perm.data[,k] - mean.prof[k]) /std.prof[k]
        pos.out[k]    <- ( pos.data[k] - mean.prof[k] )/std.prof[k]
    }
    out  <- list(pos.out = pos.out, perm.out = perm.out)
    out
}


get.class.emp.p_vals.gsz <- function(pos.data, neg.data,add=1){
    
    neg.dat.sz <- dim(neg.data)
    test_data <- rep(0, neg.dat.sz[2])
    for (k in 1:neg.dat.sz[2]){
        test_data[k] <- length(which(neg.data[,k] >= pos.data[k]))
    }
    
    test_data <- log10(neg.dat.sz[1]+add)-log10(test_data+add)
    test_data <- 10^-(test_data)
    test_data
    
    
}

