## codes used for evaluation of gene set analysis methods in mGSZm paper using mouse tissue gene expression data
## Pashupati Mishra
## Liisa Holm's Bioinformatics lab
## Institute of Biotechnology, University of Helsinki
## 28.06.2016

## loading the breast cancer data

load("brca_data_split.RData")
expr.data.brca <- brca_data$expr
gene.sets.brca <- brca_data$gene.sets
sample.labels <- brca_data$sample.labels


## indices for generating a subset of data 
set.seed(345)
iter <- 1
rand.ind <- vector("list",iter)
for(i in 1:iter){
    res <- matrix(NA,nrow=5,ncol=6)
    res[,1] <- sample(1:33,5,replace=F)
    res[,2] <- sample(34:38,5,replace=F)
    res[,3] <- sample(39:52,5,replace=F)
    res[,4] <- sample(53:75,5,replace=F)
    res[,5] <- sample(76:89,5,replace=F)
    res[,6] <- sample(90:94,5,replace=F)
    rand.ind[[i]] <- res
}


## sample labels in various formats

lbs_test <- unlist(lapply(c(1:6),function(x) rep(x,5)))
lbs_camera_test <- unlist(lapply(letters[1:6],function(x) rep(x,5)))

## load required packages

library(limma)
library(Biobase)
library(gage)
library(qusage)

# design and contrast matrix for test data for camera
des_test <- model.matrix(~0 +as.factor(lbs_camera_test))
colnames(des_test) <- levels(as.factor(lbs_camera_test))
contr_test <- makeContrasts(a-d,levels=des_test)


# data

ind <- as.numeric(rand.ind[[1]])
expr_test <- expr.data.brca[,ind]


### indices of the two compared groups ##
grp_ind <- c(1,2,3,4,5,16,17,18,19,20)

set.seed(303)
ind <- sample(1:10,5,replace=F)


### randomizing sample labels of the compared groups
grp1 <- grp_ind[ind]
grp2 <- grp_ind[-ind]
expr_rand <- expr_test[,c(grp1,6:15,grp2,21:30)]


### use working version of mGSZm

source("mGSZH_final_empPval.R")

mGSZm_obj_null <- mGSZH_final_emp(expr.data=expr_rand,gene.sets=gene.sets.brca,sample.labels=lbs_test,comparisons="group1-group4",method="m3_limma",perm.number=1000)

mGSZm_null <- mGSZm_obj_null$comparison1$mGSZH
mGSAm_null <- mGSZm_obj_null$comparison1$mGSAH
wks_null <- mGSZm_obj_null$comparison1$wksM



## romer runs ##


gene.sets.brca.mat <- toMatrix(expr_rand,gene.sets.brca)
gene.sets.brca.romer <- list()

for(i in 1:ncol(gene.sets.brca.mat)){
    ind <- which(gene.sets.brca.mat[,i]==1)
    gene.sets.brca.romer[[i]] <- ind
}

names(gene.sets.brca.romer) <- colnames(gene.sets.brca.mat)
lbs_romer <- c(rep("a",5),rep("b",5),rep("c",5),rep("d",5),rep("e",5),rep("f",5))
design <- model.matrix(~0 + as.factor(lbs_romer))
colnames(design) <- levels(as.factor(lbs_romer))
contr <- makeContrasts(a-d,levels=design)

romer_null <- romer(y=expr_rand,index=gene.sets.brca.romer,design=design,contrast=contr,nrot=1000)




## camera

gene.sets.brca.camera <- list()

for(i in 1:ncol(gene.sets.brca.mat)){
    gene.sets.brca.camera[[i]] <- as.numeric(which(gene.sets.brca.mat[,i]==1))
}
names(gene.sets.brca.camera) <- colnames(gene.sets.brca.mat)

camera_null <- camera(expr_rand,gene.sets.brca.camera,design,contrast=contr)



## qusage

gene.sets.brca.qusage <- list()
for(i in 1:ncol(gene.sets.brca.mat)){
    gene.sets.brca.qusage[[i]] <- rownames(expr_rand)[as.numeric(which(gene.sets.brca.mat[,i]==1))]
}

names(gene.sets.brca.qusage) <- colnames(gene.sets.brca.mat)

qusage_null <- qsTable(qusage(expr_rand,lbs_romer, "a-d", gene.sets.brca.qusage), number=length(gene.sets.brca.qusage))



## gage

less <- gage(expr_rand, gsets = gene.sets.brca, ref = c(1:5), samp = c(16:20), compare = "unpaired",set.size=c(1,5000))$less
greater <- gage(expr_rand, gsets = gene.sets.brca, ref = c(1:5), samp = c(16:20), compare = "unpaired",set.size=c(1,5000))$greater
greater <- greater[order(rownames(greater)),]
less <-less[order(rownames(less)),]



## plot

png("try.png",height=3,width=5,units="in",res=300)
par(mfrow = c(2, 4))
hist(mGSZm_null[,6],breaks=20,ylab="Density",xlab="P-value",main="mGSZm")
hist(10^-mGSAm_null[,6],breaks=20,ylab="Density",xlab="P-value",main="mGSA")
hist(wks_null[,6],breaks=20,ylab="Density",xlab="P-value",main="wKS")
hist(camera_null[,4],breaks=20,ylab="Density",xlab="P-value",main="CAMERA")
hist(less[,3],breaks=20,ylab="Density",xlab="P-value",main="GAGE")
hist(romer_null[,4],breaks=20,ylab="Density",xlab="P-value",main="romer")
hist(qusage_null$p.Value,ylab="Density",xlab="P-value",main="QuSAGE")
dev.off()


